/**
 * @file
 * Initializes the AutoNumeric library on formattable number fields.
 *
 * This script defines a Drupal Behavior that finds all textfields created by
 * our FormattedNumberWidget and activates the AutoNumeric input masking on them.
 */

// This is a standard JavaScript pattern called an Immediately Invoked Function
// Expression (IIFE). It creates a private scope for our code to prevent
// conflicts with other scripts on the page. We pass in global objects like
// jQuery, Drupal, once, and AutoNumeric as local variables.
(function ($, Drupal, once, AutoNumeric) {
  'use strict';

  /**
   * Drupal Behavior to apply input masking to number fields.
   *
   * Drupal's Behavior system is a way to attach JavaScript functionality to
   * elements on the page, both on initial page load and after AJAX updates.
   *
   * @type {Drupal.behavior}
   */
  Drupal.behaviors.formattedNumberInput = {
    /**
     * The attach function is called by Drupal to apply the behavior.
     *
     * @param {Element} context
     * The context is the portion of the DOM to apply the behavior to. On page
     * load it's the `document`, but after an AJAX call, it might be just the
     * new content that was loaded.
     * @param {object} settings
     * An object containing settings passed from PHP. Not used in this script.
     */
    attach: function (context) {
      // The `once()` function is a crucial part of the Drupal JS API. It ensures
      // that our code only runs *once* for each element. This prevents the
      // AutoNumeric library from being initialized multiple times on the same
      // field, which would break it.
      // We look for elements with the '.js-formattable-number' class within
      // the current `context`.
      const elements = once('formattable-number', '.js-formattable-number', context);

      // We loop through each new element found by `once()`.
      elements.forEach(function (element) {
        // A try...catch block is defensive coding. If one element has invalid
        // options and fails to initialize, it won't stop the rest of the
        // JavaScript on the page from running.
        try {
          // This is where we read the data passed from PHP.
          // `element.dataset.autonumericOptions` accesses the value of the
          // `data-autonumeric-options` attribute we set in the PHP widget.
          // `JSON.parse()` converts the JSON string back into a JavaScript object.
          const options = JSON.parse(element.dataset.autonumericOptions);
          const stylePreset = element.dataset.styleRulesPreset;

          // Add caretPositionOnFocus: null when selectOnFocus is used, to
          // prevent a conflict warning from the AutoNumeric library.
          if (options.selectOnFocus) {
            options.caretPositionOnFocus = null;
          }

          // Based on the preset selected in the widget settings, build the
          // complex styleRules object for the presets that work reliably.
          if (stylePreset && stylePreset !== 'none' && stylePreset !== 'evenOdd') {
            let styleRules = null;
            switch (stylePreset) {
              case 'positiveNegative':
                styleRules = {
                  positive: 'auto-numeric-positive',
                  negative: 'auto-numeric-negative',
                  zero: 'auto-numeric-zero',
                };
                break;

              case 'range0To100With4Steps':
                styleRules = {
                  ranges: [
                    { min: 0, max: 25, class: 'auto-numeric-red' },
                    { min: 25, max: 50, class: 'auto-numeric-orange' },
                    { min: 50, max: 75, class: 'auto-numeric-yellow' },
                    { min: 75, max: 100, class: 'auto-numeric-green' },
                  ],
                };
                break;

              case 'rangeSmallAndZero':
                // Using the reliable `ranges` option to replicate the small
                // range behavior, avoiding the userDefined callback bug.
                styleRules = {
                  ranges: [
                    { min: -0.999999999, max: -0.000000001, class: 'auto-numeric-small-negative' },
                    { min: 0, max: 0, class: 'auto-numeric-zero' },
                    { min: 0.000000001, max: 0.999999999, class: 'auto-numeric-small-positive' },
                  ],
                };
                break;
            }
            if (styleRules) {
              options.styleRules = styleRules;
            }
          }

          // Initialize AutoNumeric on the (empty) element.
          const anElement = new AutoNumeric(element, options);

          // For the 'evenOdd' preset, we bypass the unreliable styleRules
          // callback and use a direct, robust event listener instead.
          if (stylePreset === 'evenOdd') {
            element.addEventListener('autoNumeric:formatted', e => {
              const rawValue = e.detail.newRawValue;
              const target = e.target;
              if (rawValue !== null && rawValue !== '') {
                // Use Math.trunc() to check only the integer part.
                if (Math.trunc(rawValue) % 2 === 0) {
                  target.classList.add('auto-numeric-even');
                  target.classList.remove('auto-numeric-odd');
                } else {
                  target.classList.add('auto-numeric-odd');
                  target.classList.remove('auto-numeric-even');
                }
              } else {
                target.classList.remove('auto-numeric-even', 'auto-numeric-odd');
              }
            });
          }

          // Check if a raw value was passed from PHP.
          // The dataset property will be a string, so we also check that it's not empty.
          if (element.dataset.rawValue && element.dataset.rawValue !== '') {
            // Use the .set() method to programmatically set the value.
            // This bypasses any initial parsing conflicts (e.g. when a field config change
            // has happened while the edit form was loaded and correctly formats
            // the raw number according to the current options.
            anElement.set(element.dataset.rawValue);
          }

        } catch (e) {
          console.error('Error initializing AutoNumeric on element:', element, e);
        }
      });
    }
  };

}(jQuery, Drupal, once, AutoNumeric));