<?php

/**
 * @file
 * Contains \Drupal\formatted_number_input\Plugin\Field\FieldFormatter\FormattedNumberAsWidgetFormatter.
 */

namespace Drupal\formatted_number_input\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\field_ui\Form\EntityDisplayFormBase;
use Drupal\formatted_number_input\Plugin\Field\FieldWidget\FormattedNumberWidget;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Formatted as in input widget' formatter.
 *
 * This formatter is language-aware and dynamically uses the settings configured
 * for the 'formatted_number_input'-widget on a specific, user-configured form mode.
 *
 * @FieldFormatter(
 * id = "formatted_number_as_widget",
 * label = @Translation("Formatted as in input widget"),
 * field_types = {
 * "integer",
 * "decimal",
 * "float"
 * }
 * )
 */
final class FormattedNumberAsWidgetFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The entity display repository service.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  private readonly EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Constructs a new FormattedNumberAsWidgetFormatter object.
   */
  public function __construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, EntityDisplayRepositoryInterface $entity_display_repository) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * Creates an instance of the plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_display.repository')
    );
  }

  /**
   * Returns a list of default settings for the formatter.
   */
  public static function defaultSettings(): array {
    return [
      'source_form_mode' => 'default',
    ] + parent::defaultSettings();
  }

  /**
   * Builds the settings form for the formatter.
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $options = [];
    $form_object = $form_state->getFormObject();
    if ($form_object instanceof EntityDisplayFormBase) {
      $entity_type_id = $form_object->getEntity()->getTargetEntityTypeId();

      $options['default'] = $this->t('Default');

      $form_modes = $this->entityDisplayRepository->getFormModes($entity_type_id);
      foreach ($form_modes as $machine_name => $form_mode) {
        $options[$machine_name] = $form_mode['label'];
      }
    }

    $elements['source_form_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Form Mode containing the Widget Format to apply'),
      '#options' => $options,
      '#default_value' => $this->getSetting('source_form_mode'),
      '#description' => $this->t("Select the form display that contains the configured 'Formatted Number' widget. This formatter will use the settings from that widget."),
    ];

    return $elements;
  }

  /**
   * Returns a summary of the formatter's settings.
   */
  public function settingsSummary(): array {
    $summary = [];
    $source_form_mode = $this->getSetting('source_form_mode');

    if (empty($source_form_mode)) {
        $summary[] = $this->t('Source form mode not configured.');
        return $summary;
    }
    // **FIXED:** Get the form mode label safely without building a form.
    // Get the entity type from the field definition.
    $entity_type_id = $this->fieldDefinition->getTargetEntityTypeId();
      // Get all form modes for that entity type.
    $form_modes = $this->entityDisplayRepository->getFormModes($entity_type_id);
      // Manually add the 'Default' mode to the list for lookup.
    $form_modes['default'] = ['label' => $this->t('Default')];
    // Find the human-readable label for the saved machine name.
    $mode_label = $form_modes[$source_form_mode]['label'] ?? $source_form_mode;

    // Check if the correct widget is configured on the selected form mode.
    $bundle = $this->fieldDefinition->getTargetBundle();
    $field_name = $this->fieldDefinition->getName();
    $form_display = $this->entityDisplayRepository->getFormDisplay($entity_type_id, $bundle, $source_form_mode);
    $component = $form_display->getComponent($field_name);

    if (isset($component['type']) && $component['type'] === 'formatted_number_widget') {
        // Success: The correct widget is configured.
        $summary[] = $this->t('Applies language-aware settings of form mode "@mode" of formatted input widget', ['@mode' => $mode_label]);
    }
    else {
        // Warning: The widget is not configured for this field on the selected form mode.
        $widget_label = $this->t('Formatted Number Input');
        $summary[] = $this->t('⚠️ <strong>Warning:</strong> The "@widget" widget is not enabled for this field in the "@mode" form display. This formatter will have no effect.', [
            '@widget' => $widget_label,
            '@mode' => $mode_label,
        ]);
    }
    return $summary;
  }

  /**
   * Builds a renderable array for a field value.
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $source_form_mode = $this->getSetting('source_form_mode');

    if (empty($source_form_mode)) {
      foreach ($items as $delta => $item) {
        $elements[$delta] = ['#plain_text' => $item->value];
      }
      return $elements;
    }

    $widget_settings = $this->getWidgetSettings($items, $source_form_mode);

    if (!$widget_settings) {
      foreach ($items as $delta => $item) {
        $elements[$delta] = ['#plain_text' => $item->value];
      }
      return $elements;
    }

    // Attach the library that contains our CSS styles.
    $elements['#attached']['library'][] = 'formatted_number_input/formatted_number_input';

    // Merge saved widget settings with module defaults to ensure all keys exist.
    $settings = array_replace_recursive(FormattedNumberWidget::defaultSettings(), $widget_settings);
    $global_settings = $settings['global_settings'];
    $language_settings = $settings['language_settings'];

    // Get the settings for the default/fallback language first.
    $default_lang_settings = $language_settings['default'];

    // Get the settings for the current language, if they exist.
    $current_lang_settings = $language_settings[$langcode] ?? [];

    // Resolve the final settings, using the current language's value if it's
    // set, otherwise falling back to the default.
    $symbol = $current_lang_settings['symbol_' . $langcode] ?? $default_lang_settings['lang_symbol_default'];
    $symbol_placement = $current_lang_settings['symbol_place_' . $langcode] ?? $default_lang_settings['lang_symbol_placement_default'];
    $thousand_separator = $current_lang_settings['thousand_separator_' . $langcode] ?? $default_lang_settings['thousand_separator_default'];
    $decimal_separator = $current_lang_settings['decimal_separator_' . $langcode] ?? $default_lang_settings['decimal_separator_default'];
    $neg_brackets = $current_lang_settings['negativeBracketsTypeOnBlur_' . $langcode] ?? $default_lang_settings['negativeBracketsTypeOnBlur_default'];
    $neg_sign_char = $current_lang_settings['negativeSignCharacter_' . $langcode] ?? $default_lang_settings['negativeSignCharacter_default'];
    $show_pos_sign = $current_lang_settings['showPositiveSign_' . $langcode] ?? $default_lang_settings['showPositiveSign_default'];
    $pos_sign_char = $current_lang_settings['positiveSignCharacter_' . $langcode] ?? $default_lang_settings['positiveSignCharacter_default'];
    $neg_pos_sign_placement = $current_lang_settings['negativePositiveSignPlacement_' . $langcode] ?? $default_lang_settings['negativePositiveSignPlacement_default'];
    $style_preset = $global_settings['styleRules_preset'] ?? 'none';

    $decimals = $this->fieldDefinition->getSetting('scale') ?? 0;
    if ($this->fieldDefinition->getType() === 'integer') {
      $decimals = 0;
    }

    foreach ($items as $delta => $item) {
      $value = (float) $item->value;
      $is_negative = $value < 0;

      // --- 1. Calculate CSS classes based on the preset and value. ---
      $css_classes = [];
      if ($style_preset !== 'none') {
        switch ($style_preset) {
          case 'positiveNegative':
            if ($value > 0) {
              $css_classes[] = 'auto-numeric-positive';
            }
            elseif ($value < 0) {
              $css_classes[] = 'auto-numeric-negative';
            }
            else {
              $css_classes[] = 'auto-numeric-zero';
            }
            break;

          case 'range0To100With4Steps':
            if ($value >= 0 && $value < 25) {
              $css_classes[] = 'auto-numeric-red';
            }
            elseif ($value >= 25 && $value < 50) {
              $css_classes[] = 'auto-numeric-orange';
            }
            elseif ($value >= 50 && $value < 75) {
              $css_classes[] = 'auto-numeric-yellow';
            }
            elseif ($value >= 75) {
              $css_classes[] = 'auto-numeric-green';
            }
            break;

          case 'evenOdd':
            if ((int) $value % 2 === 0) {
              $css_classes[] = 'auto-numeric-even';
            }
            else {
              $css_classes[] = 'auto-numeric-odd';
            }
            break;

          case 'rangeSmallAndZero':
            if ($value > -1 && $value < 0) {
              $css_classes[] = 'auto-numeric-small-negative';
            }
            elseif ($value > 0 && $value < 1) {
              $css_classes[] = 'auto-numeric-small-positive';
            }
            elseif ($value == 0) {
              $css_classes[] = 'auto-numeric-zero';
            }
            break;
        }
      }

      // --- 2. Calculate the formatted string value. ---
      $formatted_number = number_format(abs($value), $decimals, $decimal_separator, $thousand_separator);
      $currency_prefix = '';
      $currency_suffix = '';
      if (!empty($symbol)) {
        $symbol_placement === 'p' ? $currency_prefix = rtrim($symbol) . ' ' : $currency_suffix = ' ' . ltrim($symbol);
      }

      $final_markup = '';
      // Handle accounting-style negative brackets.
      if ($is_negative && !empty($neg_brackets) && $neg_brackets !== 'none') {
        $open_bracket = $neg_brackets[0];
        $close_bracket = $neg_brackets[2];
        $placement_style = $global_settings['symbolPlacementInBrackets'] ?? 'inside';
        if ($placement_style === 'outside') {
          // Style: $ (123.45) or (123.45) €
          $final_markup = $currency_prefix . $open_bracket . $formatted_number . $close_bracket . $currency_suffix;
        }
        else {
          // Default Style: ($ 123.45) or (123,45 €)
          $final_markup = $open_bracket . $currency_prefix . $formatted_number . $currency_suffix . $close_bracket;
        }
      }
      else {
      // Handle positive/negative signs if not using brackets.
        $sign = '';
        if ($is_negative) {
          $sign = $neg_sign_char;
        }
        elseif (!empty($show_pos_sign)) {
          $sign = $pos_sign_char;
        }
        switch ($neg_pos_sign_placement) {
          case 's': // Suffix.
            $final_markup = $currency_prefix . $formatted_number . $currency_suffix . $sign;
            break;

          case 'l': // Left of number, right of prefix symbol.
            $final_markup = $currency_prefix . $sign . $formatted_number . $currency_suffix;
            break;

          case 'r': // Right of number, left of suffix symbol.
            $final_markup = $currency_prefix . $formatted_number . $sign . $currency_suffix;
            break;

          case 'p': // Prefix.
          default:
            $final_markup = $sign . $currency_prefix . $formatted_number . $currency_suffix;
            break;
        }
      }

      // --- 3. Build the final render array with classes, if any. ---
      if (!empty($css_classes)) {
        $elements[$delta] = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => $final_markup,
          '#attributes' => [
            'class' => $css_classes,
          ],
        ];
      }
      else {
        $elements[$delta] = ['#markup' => $final_markup];
      }
    }

    return $elements;
  }

  /**
   * Loads the widget settings for the current field from a specific form mode.
   */
  private function getWidgetSettings(FieldItemListInterface $items, string $form_mode): ?array {
    $entity = $items->getEntity();
    $form_display = $this->entityDisplayRepository->getFormDisplay($entity->getEntityTypeId(), $entity->bundle(), $form_mode);

    if ($form_display) {
      $component = $form_display->getComponent($items->getName());
      if (isset($component['type']) && $component['type'] === 'formatted_number_widget') {
        return $component['settings'];
      }
    }

    return NULL;
  }

}