<?php
/**
 * @file
 * Contains \Drupal\formatted_number_input\Plugin\Field\FieldWidget\FormattedNumberWidget.
 * This file is the central PHP logic for the custom field widget.
 */

namespace Drupal\formatted_number_input\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'formatted_number_widget' field widget.
 *
 * This class is a plugin, discovered by Drupal through the annotation below.
 * It's responsible for the widget's settings form, how the widget appears in
 * an entity form, and how the submitted data is processed.
 *
 * @FieldWidget(
 * id = "formatted_number_widget",
 * label = @Translation("Formatted Number Input"),
 * field_types = {
 * "integer",
 * "decimal",
 * "float"
 * }
 * )
 */
final class FormattedNumberWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  private readonly LanguageManagerInterface $languageManager;

  /**
   * Constructs a FormattedNumberWidget object.
   *
   * @param string $plugin_id
   * The plugin_id for the widget.
   * @param mixed $plugin_definition
   * The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   * The definition of the field to which the widget is associated.
   * @param array $settings
   * The widget settings.
   * @param array $third_party_settings
   * Any third-party settings.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   * The language manager.
   */
  public function __construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings, LanguageManagerInterface $language_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->languageManager = $language_manager;
  }

  /**
   * Creates an instance of the plugin.
   *
   * This static method is used by Drupal's plugin system to instantiate the
   * widget and inject the necessary services, like the Language Manager.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * The container to pull out services used in the plugin.
   * @param array $configuration
   * A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   * The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   * The plugin implementation definition.
   *
   * @return static
   * Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('language_manager')
    );
  }

  /**
   * Returns the default settings for the widget.
   *
   * This method defines the settings in a nested structure that mirrors the
   * array structure of the settings form. This allows Drupal's Form API to
   * correctly handle saving the nested data without any custom logic.
   *
   * @return array
   * An array of default settings.
   */
  public static function defaultSettings(): array {
    $defaults = [
      'global_settings' => [
        'placeholder' => '',
        'roundingMethod' => 'S',
        'formulaMode' => FALSE,
        'selectNumberOnly' => FALSE,
        'selectOnFocus' => TRUE,
        'styleRules_preset' => 'none',
        'symbolPlacementInBrackets' => 'inside',
      ],
      'language_settings' => [
        'default' => [
          'lang_symbol_default' => '',
          'lang_symbol_placement_default' => 'p',
          'thousand_separator_default' => ',',
          'decimal_separator_default' => '.',
          'negativePositiveSignPlacement_default' => 'p',
          'negativeSignCharacter_default' => '-',
          'negativeBracketsTypeOnBlur_default' => 'none',
          'positiveSignCharacter_default' => '+',
          'showPositiveSign_default' => FALSE,
        ],
      ],
    ];
    return $defaults + parent::defaultSettings();
  }

  /**
   * A helper function to ensure all settings have default values.
   *
   * @return array
   * The merged settings.
   */
  private function getMergedSettings(): array {
    // Manually merge the settings with the defaults to ensure all keys exist.
    // This is the most robust way to prevent "Undefined array key" warnings
    // for widgets that were configured before a new setting was added.
    $defaults = static::defaultSettings();
    return array_replace_recursive($defaults, $this->getSettings());
  }

  /**
   * Returns the form for the widget settings.
   *
   * This form appears on the "Manage form display" page. This implementation
   * uses a simple nested PHP array. This structure renders correctly and allows
   * Drupal's Form API to natively handle the "Update" and "Save" flow for
   * nested settings without requiring any custom validators or submit handlers.
   *
   * @param array $form
   * The form where the settings form is being included in.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * The current state of the form.
   *
   * @return array
   * The widget settings form elements.
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $elements = [];
    $settings = $this->getMergedSettings();
    $global_settings = $settings['global_settings'];

    // Load the helper function and get language-specific defaults.
    $module_handler = \Drupal::moduleHandler();
    if ($module_handler->moduleExists('formatted_number_input')) {
      $module_handler->load('formatted_number_input');
    }
    $all_language_defaults = function_exists('formatted_number_input_get_language_defaults') ? formatted_number_input_get_language_defaults() : [];

    // Get the site's default language to provide a smart fallback default.
    $site_default_langcode = $this->languageManager->getDefaultLanguage()->getId();
    $suggested_fallback_defaults = $all_language_defaults[$site_default_langcode] ?? [];

    // Global settings are grouped in a nested array.
    $elements['global_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Global Settings'),
      '#open' => TRUE,
    ];
    $elements['global_settings']['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $global_settings['placeholder'],
    ];

    $elements['global_settings']['roundingMethod'] = [
      '#type' => 'select',
      '#title' => $this->t('Rounding Method'),
      '#options' => [
        'S' => $this->t("Round-Half-Up Symmetric (default)"),
        'A' => $this->t("Round-Half-Up Asymmetric"),
        's' => $this->t("Round-Half-Down Symmetric"),
        'a' => $this->t("Round-Half-Down Asymmetric"),
        'B' => $this->t("Round-Half-Even (Bankers)"),
        'U' => $this->t("Round Up (away from zero)"),
        'D' => $this->t("Round Down (towards zero)"),
        'C' => $this->t("Round to Ceiling (towards positive infinity)"),
        'F' => $this->t("Round to Floor (towards negative infinity)"),
        'CHF' => $this->t("Round to nearest .05 (for CHF)"),
      ],
      '#default_value' => $global_settings['roundingMethod'],
      '#description' => $this->t('Defines the rounding method used. See AutoNumeric documentation for details.'),
    ];
    $elements['global_settings']['formulaMode'] = [
      '#type' => 'select',
      '#title' => $this->t('Formula Mode'),
      '#options' => [0 => $this->t('No'), 1 => $this->t('Yes')],
      '#default_value' => $global_settings['formulaMode'],
      '#description' => $this->t('If enabled, users can enter simple formulas directly in the input (e.g., "15*3").'),
    ];
    $elements['global_settings']['selectNumberOnly'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Number Only'),
      '#options' => [0 => $this->t('No'), 1 => $this->t('Yes')],
      '#default_value' => $global_settings['selectNumberOnly'],
      '#description' => $this->t('If enabled, selecting the input content will only select the number, not the currency symbol or separators.'),
    ];
    $elements['global_settings']['selectOnFocus'] = [
      '#type' => 'select',
      '#title' => $this->t('Select on Focus'),
      '#options' => [1 => $this->t('Yes'), 0 => $this->t('No')],
      '#default_value' => $global_settings['selectOnFocus'],
      '#description' => $this->t('If enabled, the entire input value will be selected when it receives focus.'),
    ];

    $elements['global_settings']['styleRules_preset'] = [
      '#type' => 'select',
      '#title' => $this->t('Style Rules Preset'),
      '#options' => [
        'none' => $this->t('None'),
        'positiveNegative' => $this->t('Positive / Negative / Zero'),
        'range0To100With4Steps' => $this->t('0-100 Color Range (4 Steps)'),
        'evenOdd' => $this->t('Even / Odd'),
        'rangeSmallAndZero' => $this->t('Small Range Around Zero'),
      ],
      '#default_value' => $global_settings['styleRules_preset'],
      '#description' => $this->t('Select a preset to apply CSS classes based on the input value. The module provides default CSS for these presets.'),
    ];

    $elements['global_settings']['symbolPlacementInBrackets'] = [
      '#type' => 'select',
      '#title' => $this->t('Symbol Placement for Bracketed Negatives'),
      '#options' => [
        'inside' => $this->t('Inside Brackets, e.g., ($123.45)'),
        'outside' => $this->t('Outside Brackets, e.g., $(123.45)'),
      ],
      '#default_value' => $global_settings['symbolPlacementInBrackets'],
      '#description' => $this->t('This only affects the display formatter, not the input widget itself.'),
    ];

    // Language settings are in their own nested group.
    $elements['language_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Formatted Number Input Settings'),
      '#open' => TRUE,
    ];

    // Default (Fallback) fieldset.
    $elements['language_settings']['default'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Default (Fallback)'),
    ];

    // Use saved settings, or suggested fallback defaults, or hardcoded defaults.
    $saved_fallback_settings = $settings['language_settings']['default'] ?? [];
    $elements['language_settings']['default']['lang_symbol_default'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Unit or Currency Symbol for this language'),
      '#default_value' => $saved_fallback_settings['lang_symbol_default'] ?? $suggested_fallback_defaults['symbol'] ?? '',
      '#size' => 5,
    ];
    $elements['language_settings']['default']['lang_symbol_placement_default'] = [
      '#type' => 'select',
      '#title' => $this->t('Symbol placement as prefix or postfix'),
      '#options' => ['p' => $this->t('Prefix'), 's' => $this->t('Suffix')],
      '#default_value' => $saved_fallback_settings['lang_symbol_placement_default'] ?? $suggested_fallback_defaults['symbol_placement'] ?? 'p',
    ];
    $elements['language_settings']['default']['thousand_separator_default'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Thousand separator'),
      '#default_value' => $saved_fallback_settings['thousand_separator_default'] ?? $suggested_fallback_defaults['thousand_separator'] ?? ',',
      '#size' => 5,
    ];
    $elements['language_settings']['default']['decimal_separator_default'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Decimal separator'),
      '#default_value' => $saved_fallback_settings['decimal_separator_default'] ?? $suggested_fallback_defaults['decimal_separator'] ?? '.',
      '#size' => 5,
    ];
    // New accounting settings for default/fallback.
    $elements['language_settings']['default']['negativePositiveSignPlacement_default'] = [
      '#type' => 'select',
      '#title' => $this->t('Sign Placement'),
      '#options' => [
        'p' => $this->t('Prefix (before currency symbol)'),
        's' => $this->t('Suffix (after currency symbol)'),
        'l' => $this->t('Left (of number, right of prefix currency symbol)'),
        'r' => $this->t('Right (of number, left of suffix currency symbol)'),
      ],
      '#default_value' => $saved_fallback_settings['negativePositiveSignPlacement_default'] ?? $suggested_fallback_defaults['negativePositiveSignPlacement'] ?? 'p',
    ];
    $elements['language_settings']['default']['negativeSignCharacter_default'] = [
      '#type' => 'select',
      '#title' => $this->t('Negative Sign Character'),
      '#options' => ['-' => '-', '–' => '–'],
      '#default_value' => $saved_fallback_settings['negativeSignCharacter_default'] ?? $suggested_fallback_defaults['negativeSignCharacter'] ?? '-',
    ];
    $elements['language_settings']['default']['negativeBracketsTypeOnBlur_default'] = [
      '#type' => 'select',
      '#title' => $this->t('Negative Value Brackets'),
      '#options' => [
        'none' => $this->t('None'),
        '(,)' => '(...)',
        '[,]' => '[...]',
        '{,}' => '{...}',
        '<,>' => '<...>',
      ],
      '#description' => $this->t('Wraps negative numbers in brackets on blur. Overrides sign placement.'),
      '#default_value' => $saved_fallback_settings['negativeBracketsTypeOnBlur_default'] ?? $suggested_fallback_defaults['negativeBracketsTypeOnBlur'] ?? 'none',
    ];
    $elements['language_settings']['default']['positiveSignCharacter_default'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Positive Sign Character'),
      '#default_value' => $saved_fallback_settings['positiveSignCharacter_default'] ?? $suggested_fallback_defaults['positiveSignCharacter'] ?? '+',
      '#size' => 5,
    ];
    $elements['language_settings']['default']['showPositiveSign_default'] = [
      '#type' => 'select',
      '#title' => $this->t('Show Positive Sign'),
      '#options' => [0 => $this->t('No'), 1 => $this->t('Yes')],
      '#default_value' => $saved_fallback_settings['showPositiveSign_default'] ?? $suggested_fallback_defaults['showPositiveSign'] ?? 0,
    ];

    // Read the nested settings for language-specific values.
    $lang_settings = $settings['language_settings'];
    $default_symbol_lang = $lang_settings['default']['lang_symbol_default'] ?? '';
    $default_symbol_placement_lang = $lang_settings['default']['lang_symbol_placement_default'] ?? 'p';
    $default_thousand_sep = $lang_settings['default']['thousand_separator_default'] ?? ',';
    $default_decimal_sep = $lang_settings['default']['decimal_separator_default'] ?? '.';
    // New fallback defaults.
    $default_neg_pos_sign_placement = $lang_settings['default']['negativePositiveSignPlacement_default'] ?? 'p';
    $default_neg_sign_char = $lang_settings['default']['negativeSignCharacter_default'] ?? '-';
    $default_neg_brackets = $lang_settings['default']['negativeBracketsTypeOnBlur_default'] ?? 'none';
    $default_pos_sign_char = $lang_settings['default']['positiveSignCharacter_default'] ?? '+';
    $default_show_pos_sign = $lang_settings['default']['showPositiveSign_default'] ?? 0;

    foreach ($this->languageManager->getLanguages() as $langcode => $language) {
      $elements['language_settings'][$langcode] = [
        '#type' => 'fieldset',
        '#title' => $this->t('@language (@langcode)', ['@language' => $language->getName(), '@langcode' => $langcode]),
      ];
      $suggested_lang_defaults = $all_language_defaults[$langcode] ?? [];
      $saved_lang_settings = $lang_settings[$langcode] ?? [];
      // Use saved setting, or suggested language default, or empty/fallback.
      $key_symbol = 'symbol_' . $langcode;
      $elements['language_settings'][$langcode][$key_symbol] = [
        '#type' => 'textfield',
        '#title' => $this->t('Symbol'),
        '#default_value' => $saved_lang_settings[$key_symbol] ?? $suggested_lang_defaults['symbol'] ?? '',
        '#placeholder' => $default_symbol_lang,
        '#size' => 5,
      ];
      $key_symbol_placement = 'symbol_place_' . $langcode;
      $elements['language_settings'][$langcode][$key_symbol_placement] = [
        '#type' => 'select',
        '#title' => $this->t('Symbol Placement'),
        '#options' => ['p' => $this->t('Prefix'), 's' => $this->t('Suffix')],
        '#default_value' => $saved_lang_settings[$key_symbol_placement] ?? $suggested_lang_defaults['symbol_placement'] ?? 'p',
      ];
      $key_thousand = 'thousand_separator_' . $langcode;
      $elements['language_settings'][$langcode][$key_thousand] = [
        '#type' => 'textfield',
        '#title' => $this->t('Thousand separator'),
        '#default_value' => $saved_lang_settings[$key_thousand] ?? $suggested_lang_defaults['thousand_separator'] ?? '',
        '#placeholder' => $default_thousand_sep,
        '#size' => 5,
      ];
      $key_decimal = 'decimal_separator_' . $langcode;
      $elements['language_settings'][$langcode][$key_decimal] = [
        '#type' => 'textfield',
        '#title' => $this->t('Decimal separator'),
        '#default_value' => $saved_lang_settings[$key_decimal] ?? $suggested_lang_defaults['decimal_separator'] ?? '',
        '#placeholder' => $default_decimal_sep,
        '#size' => 5,
      ];
      // New per-language accounting settings.
      $key_neg_pos_sign_placement = 'negativePositiveSignPlacement_' . $langcode;
      $elements['language_settings'][$langcode][$key_neg_pos_sign_placement] = [
        '#type' => 'select',
        '#title' => $this->t('Sign Placement'),
        '#options' => [
          'p' => $this->t('Prefix'),
          's' => $this->t('Suffix'),
          'l' => $this->t('Left'),
          'r' => $this->t('Right'),
        ],
        '#default_value' => $saved_lang_settings[$key_neg_pos_sign_placement] ?? $suggested_lang_defaults['negativePositiveSignPlacement'] ?? $default_neg_pos_sign_placement,
      ];
      $key_neg_sign_char = 'negativeSignCharacter_' . $langcode;
      $elements['language_settings'][$langcode][$key_neg_sign_char] = [
        '#type' => 'select',
        '#title' => $this->t('Negative Sign Character'),
        '#options' => ['-' => '-', '–' => '–'],
        '#default_value' => $saved_lang_settings[$key_neg_sign_char] ?? $suggested_lang_defaults['negativeSignCharacter'] ?? $default_neg_sign_char,
      ];
      $key_neg_brackets = 'negativeBracketsTypeOnBlur_' . $langcode;
      $elements['language_settings'][$langcode][$key_neg_brackets] = [
        '#type' => 'select',
        '#title' => $this->t('Negative Value Brackets'),
        '#options' => [
          'none' => $this->t('None'),
          '(,)' => '(...)',
          '[,]' => '[...]',
          '{,}' => '{...}',
          '<,>' => '<...>',
        ],
        '#default_value' => $saved_lang_settings[$key_neg_brackets] ?? $suggested_lang_defaults['negativeBracketsTypeOnBlur'] ?? $default_neg_brackets,
      ];
      $key_pos_sign_char = 'positiveSignCharacter_' . $langcode;
      $elements['language_settings'][$langcode][$key_pos_sign_char] = [
        '#type' => 'textfield',
        '#title' => $this->t('Positive Sign Character'),
        '#default_value' => $saved_lang_settings[$key_pos_sign_char] ?? $suggested_lang_defaults['positiveSignCharacter'] ?? '',
        '#placeholder' => $default_pos_sign_char,
        '#size' => 5,
      ];
      $key_show_pos_sign = 'showPositiveSign_' . $langcode;
      $elements['language_settings'][$langcode][$key_show_pos_sign] = [
        '#type' => 'select',
        '#title' => $this->t('Show Positive Sign'),
        '#options' => [0 => $this->t('No'), 1 => $this->t('Yes')],
        '#default_value' => $saved_lang_settings[$key_show_pos_sign] ?? $suggested_lang_defaults['showPositiveSign'] ?? $default_show_pos_sign,
      ];
    }
    return $elements;
  }

  /**
   * Returns a short summary for the widget settings.
   *
   * This summary is displayed on the "Manage form display" screen, giving the
   * site builder a quick overview of the current configuration by reading from
   * the nested settings structure. This version shows both the default fallback
   * and the resolved settings for each available language.
   *
   * @return array
   * An array of summary strings.
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getMergedSettings();
    $global_settings = $settings['global_settings'];
    $language_settings = $settings['language_settings'];

    // Display placeholder if set.
    if (!empty($global_settings['placeholder'])) {
      $summary[] = $this->t('Placeholder: "@placeholder"', ['@placeholder' => $global_settings['placeholder']]);
    }

    $rounding_methods = [
      'S' => $this->t("Round-Half-Up Symmetric"),
      'A' => $this->t("Round-Half-Up Asymmetric"),
      's' => $this->t("Round-Half-Down Symmetric"),
      'a' => $this->t("Round-Half-Down Asymmetric"),
      'B' => $this->t("Round-Half-Even (Bankers)"),
      'U' => $this->t("Round Up"),
      'D' => $this->t("Round Down"),
      'C' => $this->t("Ceiling"),
      'F' => $this->t("Floor"),
      'CHF' => $this->t("CHF Rounding"),
    ];

    $rounding_label = $rounding_methods[$global_settings['roundingMethod']];
    $summary[] = $this->t('Rounding: @method', ['@method' => $rounding_label]);

    $symbol_placement_labels = [
      'inside' => $this->t('Inside'),
      'outside' => $this->t('Outside'),
    ];
    $bracket_placement_label = $symbol_placement_labels[$global_settings['symbolPlacementInBrackets']] ?? $this->t('Inside');
    $summary[] = $this->t('Bracketed symbol placement: @placement', ['@placement' => $bracket_placement_label]);

    $summary[] = $this->t('Formula mode: @mode', ['@mode' => !empty($global_settings['formulaMode']) ? $this->t('Yes') : $this->t('No')]);
    $summary[] = $this->t('Select number only: @select', ['@select' => !empty($global_settings['selectNumberOnly']) ? $this->t('Yes') : $this->t('No')]);
    $summary[] = $this->t('Select on focus: @select', ['@select' => !empty($global_settings['selectOnFocus']) ? $this->t('Yes') : $this->t('No')]);

    $preset_options = [
      'none' => $this->t('None'),
      'positiveNegative' => $this->t('Positive / Negative / Zero'),
      'range0To100With4Steps' => $this->t('0-100 Color Range (4 Steps)'),
      'evenOdd' => $this->t('Even / Odd'),
      'rangeSmallAndZero' => $this->t('Small Range Around Zero'),
    ];
    $summary[] = $this->t('Style preset: @preset', ['@preset' => $preset_options[$global_settings['styleRules_preset']]]);

    // Define default values from settings.
    $default_symbol = $language_settings['default']['lang_symbol_default'] ?? '';
    $default_placement_code = $language_settings['default']['lang_symbol_placement_default'] ?? 'p';
    $default_thousand_sep = $language_settings['default']['thousand_separator_default'] ?? ',';
    $default_decimal_sep = $language_settings['default']['decimal_separator_default'] ?? '.';
    $placement = ($default_placement_code === 'p') ? $this->t('Prefix') : $this->t('Suffix');
    $default_sample_number = '1' . $default_thousand_sep . '000' . $default_decimal_sep . '00';
    $default_sample = ($default_placement_code === 'p') ? $default_symbol . " " . $default_sample_number : $default_sample_number . " " . $default_symbol;
    // New default accounting settings.
    $default_neg_brackets = $language_settings['default']['negativeBracketsTypeOnBlur_default'] ?? 'none';
    $default_show_pos_sign = !empty($language_settings['default']['showPositiveSign_default']);

    // Build the summary line according to the user's requested format.
    $summary[] = $this->t('DEFAULT number format: Symbol "@symbol" (@placement) Thousand "@t" Decimal "@d" e.g. "@sampleno" ', [
      '@symbol' => $default_symbol ?: 'none',
      '@placement' => $placement,
      '@t' => $default_thousand_sep,
      '@d' => $default_decimal_sep,
      '@sampleno' => $default_sample,
    ]);
    $summary[] = $this->t('DEFAULT accounting: Negative brackets: @brackets, Show positive sign: @show_pos_sign', [
      '@brackets' => $default_neg_brackets,
      '@show_pos_sign' => $default_show_pos_sign ? $this->t('Yes') : $this->t('No'),
    ]);

    // Display the effective configuration for each language.
    foreach ($this->languageManager->getLanguages() as $langcode => $language) {
      $key_symbol = 'symbol_' . $langcode;
      $key_symbol_placement = 'symbol_place_' . $langcode;
      $key_thousand = 'thousand_separator_' . $langcode;
      $key_decimal = 'decimal_separator_' . $langcode;
      // New language keys.
      $key_neg_brackets = 'negativeBracketsTypeOnBlur_' . $langcode;
      $key_show_pos_sign = 'showPositiveSign_' . $langcode;

      // Resolve settings: use language-specific value if present, otherwise
      // fall back to the default.
      $final_symbol = !empty($language_settings[$langcode][$key_symbol]) ? $language_settings[$langcode][$key_symbol] : $default_symbol;
      $final_placement_code = $language_settings[$langcode][$key_symbol_placement] ?? $default_placement_code;
      $final_thousand_sep = !empty($language_settings[$langcode][$key_thousand]) ? $language_settings[$langcode][$key_thousand] : $default_thousand_sep;
      $final_decimal_sep = !empty($language_settings[$langcode][$key_decimal]) ? $language_settings[$langcode][$key_decimal] : $default_decimal_sep;
      // Resolve new accounting settings.
      $final_neg_brackets = $language_settings[$langcode][$key_neg_brackets] ?? $default_neg_brackets;
      $final_show_pos_sign = !empty($language_settings[$langcode][$key_show_pos_sign] ?? $default_show_pos_sign);

      $placement_label = ($final_placement_code === 'p') ? $this->t('Prefix') : $this->t('Suffix');
      $placement_sample_number = '1' . $final_thousand_sep . '000' . $final_decimal_sep . '00';
      $placement_sample = ($final_placement_code === 'p') ? $final_symbol . " " . $placement_sample_number : $placement_sample_number . " " . $final_symbol;
      $accounting_summary = $this->t('Neg. brackets: @brackets, Pos. sign: @show_pos_sign', [
        '@brackets' => $final_neg_brackets,
        '@show_pos_sign' => $final_show_pos_sign ? $this->t('Yes') : $this->t('No'),
      ]);

      $summary[] = $this->t('@name (@code): "@sampleno" / @acct', [
        '@name' => $language->getName(),
        '@code' => $langcode,
        '@sampleno' => $placement_sample,
        '@acct' => $accounting_summary,
      ]);
    }

    return $summary;
  }

  /**
   * Returns the form element for a single field item.
   *
   * To handle the edge case where form settings are changed while a content
   * entry form is open, this method renders the input field empty and passes
   * the raw database value in a 'data-raw-value' attribute. JavaScript then
   * uses this raw value to format the number on the client side using the
   * most up-to-date settings.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   * The items from the field to render.
   * @param int $delta
   * The delta of the item to render.
   * @param array $element
   * The FAPI element made by the widget.
   * @param array $form
   * The form structure where widgets are being attached to.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * The form state.
   *
   * @return array
   * The FAPI element for the field item.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $settings = $this->getMergedSettings();
    $field_settings = $this->fieldDefinition->getSettings();
    $global_settings = $settings['global_settings'];
    $language_settings = $settings['language_settings'];
    // Get the raw, unformatted numeric value from the database.
    $raw_value = $items[$delta]->value;

    $langcode = $items->getLangcode();

    $key_symbol = 'symbol_' . $langcode;
    $key_symbol_placement = 'symbol_place_' . $langcode;
    $key_thousand = 'thousand_separator_' . $langcode;
    $key_decimal = 'decimal_separator_' . $langcode;
    // New accounting keys.
    $key_neg_pos_sign_placement = 'negativePositiveSignPlacement_' . $langcode;
    $key_neg_sign_char = 'negativeSignCharacter_' . $langcode;
    $key_neg_brackets = 'negativeBracketsTypeOnBlur_' . $langcode;
    $key_pos_sign_char = 'positiveSignCharacter_' . $langcode;
    $key_show_pos_sign = 'showPositiveSign_' . $langcode;

    // Prepare default values.
    $default_symbol_lang = $language_settings['default']['lang_symbol_default'] ?? '';
    $default_symbol_placement_lang = $language_settings['default']['lang_symbol_placement_default'] ?? 'p';
    $default_thousand_sep = $language_settings['default']['thousand_separator_default'] ?? ',';
    $default_decimal_sep = $language_settings['default']['decimal_separator_default'] ?? '.';
    // New default values.
    $default_neg_pos_sign_placement = $language_settings['default']['negativePositiveSignPlacement_default'] ?? 'p';
    $default_neg_sign_char = $language_settings['default']['negativeSignCharacter_default'] ?? '-';
    $default_neg_brackets = $language_settings['default']['negativeBracketsTypeOnBlur_default'] ?? 'none';
    $default_pos_sign_char = $language_settings['default']['positiveSignCharacter_default'] ?? '+';
    $default_show_pos_sign = $language_settings['default']['showPositiveSign_default'] ?? 0;

    // Resolve settings by checking for a language-specific value, then falling
    // back to the default.
    $symbol_lang = !empty($language_settings[$langcode][$key_symbol]) ? $language_settings[$langcode][$key_symbol] : $default_symbol_lang;
    $symbol_lang_placement = $language_settings[$langcode][$key_symbol_placement] ?? $default_symbol_placement_lang;
    $thousand_separator = !empty($language_settings[$langcode][$key_thousand]) ? $language_settings[$langcode][$key_thousand] : $default_thousand_sep;
    $decimal_separator = !empty($language_settings[$langcode][$key_decimal]) ? $language_settings[$langcode][$key_decimal] : $default_decimal_sep;
    // Resolve new accounting settings.
    $neg_pos_sign_placement = $language_settings[$langcode][$key_neg_pos_sign_placement] ?? $default_neg_pos_sign_placement;
    $neg_sign_char = $language_settings[$langcode][$key_neg_sign_char] ?? $default_neg_sign_char;
    $neg_brackets = $language_settings[$langcode][$key_neg_brackets] ?? $default_neg_brackets;
    $pos_sign_char = !empty($language_settings[$langcode][$key_pos_sign_char]) ? $language_settings[$langcode][$key_pos_sign_char] : $default_pos_sign_char;
    $show_pos_sign = $language_settings[$langcode][$key_show_pos_sign] ?? $default_show_pos_sign;

    $element['value'] = $element + [
      '#type' => 'textfield',
      // Set #default_value to NULL. The value will be populated by JavaScript.
      // This prevents parsing conflicts if settings change while form is open.
      '#default_value' => NULL,
      '#placeholder' => $global_settings['placeholder'],
      '#element_validate' => [[static::class, 'validateAndUnformat']],
      // The validation callback receives a simple, flat array of settings.
      '#number_widget_settings' => [
        'language_symbol' => $symbol_lang,
        'thousand_separator' => $thousand_separator,
        'decimal_separator' => $decimal_separator,
        // Pass new settings to the validator.
        'negativeBracketsTypeOnBlur' => $neg_brackets,
        'positiveSignCharacter' => $pos_sign_char,
        'negativeSignCharacter' => $neg_sign_char,
      ],
      '#attached' => ['library' => ['formatted_number_input/formatted_number_input']],
    ];

    $autonumeric_options = [
      'digitGroupSeparator' => $thousand_separator,
      'decimalCharacter' => $decimal_separator,
      'currencySymbol' => $symbol_lang,
      'currencySymbolPlacement' => $symbol_lang_placement,
      'decimalPlaces' => $this->fieldDefinition->getType() === 'integer' ? 0 : ($field_settings['scale'] ?? 2),
      'modifyValueOnWheel' => FALSE,
      'roundingMethod' => $global_settings['roundingMethod'],
      'formulaMode' => !empty($global_settings['formulaMode']),
      'selectNumberOnly' => !empty($global_settings['selectNumberOnly']),
      'selectOnFocus' => !empty($global_settings['selectOnFocus']),
      // Add accounting options.
      'negativePositiveSignPlacement' => $neg_pos_sign_placement,
      'negativeSignCharacter' => $neg_sign_char,
      'negativeBracketsTypeOnBlur' => ($neg_brackets === 'none') ? NULL : $neg_brackets,
      'positiveSignCharacter' => $pos_sign_char,
      'showPositiveSign' => !empty($show_pos_sign),
    ];

    // BEGIN: Added code to respect field storage constraints.
    $field_type = $this->fieldDefinition->getType();
    switch ($field_type) {
      case 'decimal':
        $precision = $this->fieldDefinition->getSetting('precision') ?? 10;
        $scale = $this->fieldDefinition->getSetting('scale') ?? 2;
        $integer_part_length = $precision - $scale;

        if ($integer_part_length > 0) {
          $max_integer_part = str_repeat('9', $integer_part_length);

          if ($scale > 0) {
            $max_decimal_part = str_repeat('9', $scale);
            $max = "{$max_integer_part}.{$max_decimal_part}";
          }
          else {
            $max = $max_integer_part;
          }

          $autonumeric_options['maximumValue'] = $max;

          if (!empty($this->fieldDefinition->getSetting('unsigned'))) {
            $autonumeric_options['minimumValue'] = '0';
          }
          else {
            $autonumeric_options['minimumValue'] = "-{$max}";
          }
        }
        break;

      case 'integer':
      case 'float':
        $min_setting = $this->fieldDefinition->getSetting('min');
        $max_setting = $this->fieldDefinition->getSetting('max');

        if ($min_setting !== NULL && $min_setting !== '') {
          $autonumeric_options['minimumValue'] = (string) $min_setting;
        }
        if ($max_setting !== NULL && $max_setting !== '') {
          $autonumeric_options['maximumValue'] = (string) $max_setting;
        }
        break;
    }
    // END: Added code.

    // Add the raw database value and options to the element's attributes so
    // the JavaScript can access them.
    $element['value']['#attributes'] = [
      'data-autonumeric-options' => json_encode($autonumeric_options),
      // Pass the raw value to the JS for client-side formatting.
      'data-raw-value' => $raw_value,
      'class' => ['js-formattable-number'],
      // Pass the preset name to the JavaScript.
      'data-style-rules-preset' => $global_settings['styleRules_preset'],
    ];

    return $element;
  }

  /**
   * Custom element validation callback to un-format the submitted value.
   *
   * This static method is called during form submission for the content entry
   * form. Its job is to reverse the formatting applied by the JavaScript
   * library, converting a string like "$1,250.75" back into a raw numeric
   * string like "1250.75" that Drupal can save to the database.
   *
   * @param array $element
   * The form element to validate.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * The current state of the form.
   * @param array $complete_form
   * The complete form array.
   */
  public static function validateAndUnformat(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    $value = $element['#value'];
    $settings = $element['#number_widget_settings'];
    $unformatted_value = self::unformatValue($value, $settings);
    $form_state->setValueForElement($element, $unformatted_value);
  }

  /**
   * A public, testable method to unformat a value based on settings.
   *
   * This method contains the pure data transformation logic. By keeping it
   * separate from the Form API details, we can easily write a Unit Test for it.
   *
   * @param string|null $value
   * The formatted string from user input (e.g., "$1,234.56").
   * @param array $settings
   * The widget settings array, containing resolved separators.
   *
   * @return string|null
   * The unformatted, raw numeric string (e.g., "1234.56").
   */
  public static function unformatValue(?string $value, array $settings): ?string {
    if ($value === '' || $value === NULL) {
      return NULL;
    }

    $working_value = trim($value);
    $is_negative = FALSE;

    // Step 1: Detect negativity from brackets and strip them.
    if (!empty($settings['negativeBracketsTypeOnBlur']) && $settings['negativeBracketsTypeOnBlur'] !== 'none') {
      // **THE FIX IS HERE**: Correctly get the first and third characters of
      // the setting string (e.g., from '(,)' get '(' and ')').
      $bracket_setting = $settings['negativeBracketsTypeOnBlur'];
      $open_bracket = $bracket_setting[0];
      $close_bracket = $bracket_setting[2];

      if (str_starts_with($working_value, $open_bracket) && str_ends_with($working_value, $close_bracket)) {
        $is_negative = TRUE;
        // Strip only the outer brackets to continue processing the inside.
        $working_value = substr($working_value, 1, -1);
      }
    }

    // Step 2: As a fallback, detect negativity from the sign character.
    if (!$is_negative && !empty($settings['negativeSignCharacter'])) {
      if (str_contains(trim($working_value), $settings['negativeSignCharacter'])) {
        $is_negative = TRUE;
      }
    }

    // Step 3: Remove all remaining formatting except digits and the decimal
    // separator from the string (which may have had brackets removed).
    $decimal_separator = $settings['decimal_separator'] ?? '.';
    $regex = '/[^\d' . preg_quote((string) $decimal_separator, '/') . ']/u';
    $cleaned_value = preg_replace($regex, '', $working_value);

    // Step 4: Standardize the decimal separator to a period.
    if (!empty($decimal_separator) && $decimal_separator !== '.') {
      $cleaned_value = str_replace($decimal_separator, '.', $cleaned_value);
    }

    // Step 5: Add the standard negative sign if the number was flagged.
    if ($is_negative) {
      return '-' . $cleaned_value;
    }

    return $cleaned_value;
  }

}