<?php

declare(strict_types=1);

namespace Drupal\Tests\formatted_number_input\Kernel;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Render\RendererInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\node\NodeTypeInterface;

/**
 * Kernel tests for the formatted_number_input widget and formatter.
 *
 * This test verifies that the widget's settings are correctly used by the
 * display formatter to render the final output.
 *
 * @group formatted_number_input
 */
final class FormattedNumberFieldTest extends EntityKernelTestBase {

  /**
   * The node type used for testing.
   */
  protected NodeTypeInterface $nodeType;

  /**
   * The name of the test field.
   */
  protected string $fieldName = 'field_test_number';

  /**
   * The field definition used for testing.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  protected FieldConfig $fieldDefinition;

  /**
   * The renderer service.
   */
  protected RendererInterface $renderer;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'field',
    'text',
    'user',
    'system',
    'language',
    'locale',
    'config_translation',
    'content_translation',
    'formatted_number_input',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Set up the necessary database schemas and configuration.
    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installSchema('locale', ['locales_source', 'locales_target', 'locales_location']);
    $this->installConfig(['system', 'field', 'node', 'language', 'locale', 'config_translation', 'content_translation']);

    ConfigurableLanguage::createFromLangcode('de')->save();
    ConfigurableLanguage::createFromLangcode('fr')->save();

    $this->nodeType = NodeType::create([
      'type' => 'test_page',
      'name' => 'Test Page',
    ]);
    $this->nodeType->save();

    $this->container->get('content_translation.manager')
      ->setEnabled('node', 'test_page', TRUE);

    // Create a decimal field storage configuration.
    // This defines the database columns for our field.

    FieldStorageConfig::create([
      'field_name' => $this->fieldName,
      'entity_type' => 'node',
      'type' => 'decimal',
      'settings' => ['precision' => 10, 'scale' => 2],
    ])->save();

    // Create the field instance and store it in our class property.
    // This object is the FieldDefinitionInterface required by the widget.
    $this->fieldDefinition = FieldConfig::create([
      'field_name' => $this->fieldName,
      'entity_type' => 'node',
      'bundle' => 'test_page',
      'label' => 'Test Number Field',
      'translatable' => TRUE,
    ]);
    $this->fieldDefinition->save();

    $this->renderer = $this->container->get('renderer');
  }

  /**
   * Tests the formatter's output based on various widget settings.
   *
   * @dataProvider providerFormatterCases
   */
  public function testFormatterOutput(array $widgetSettings, float $value, string $langcode, string $expectedOutput): void {
    // 1. CONFIGURE THE WIDGET AND FORM DISPLAY
    // Merging with default settings is crucial to pass schema checks.
    $form_display_settings = array_replace_recursive($this->getDisplayComponentDefaults(), $widgetSettings);
    $form_display = EntityFormDisplay::create([
      'targetEntityType' => 'node',
      'bundle' => 'test_page',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $form_display->setComponent($this->fieldName, [
      'type' => 'formatted_number_widget',
      'settings' => $form_display_settings,
    ]);
    $form_display->save();

    // 2. CONFIGURE THE FORMATTER AND VIEW DISPLAY
    // Get the view display and configure the formatter to use our custom one.
    $view_display = EntityViewDisplay::create([
      'targetEntityType' => 'node',
      'bundle' => 'test_page',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $view_display->setComponent($this->fieldName, [
      'type' => 'formatted_number_as_widget',
      'label' => 'hidden',
      'settings' => [
        'source_form_mode' => 'default',
      ],
    ]);
    $view_display->save();

    // 3. CREATE CONTENT
    // Create a new node with the test value.
    $node = Node::create([
      'type' => 'test_page',
      'title' => 'Test Node',
      'langcode' => $langcode,
      $this->fieldName => $value,
    ]);
    $node->save();

    // 4. RENDER AND ASSERT
    $build = $view_display->build($node);
    $field_output = $this->renderer->renderRoot($build[$this->fieldName]);

    $this->assertStringContainsString($expectedOutput, (string) $field_output);
  }

  /**
   * Data provider for testFormatterOutput().
   */
  public static function providerFormatterCases(): array {
    return [
      'US Positive Number' => [
        'widgetSettings' => self::createWidgetSettings('en', [
          'symbol' => '$',
          'symbol_place' => 'p',
          'thousand_separator' => ',',
          'decimal_separator' => '.',
        ]),
        'value' => 12345.67,
        'langcode' => 'en',
        'expectedOutput' => '$ 12,345.67',
      ],
      'German Negative with Brackets (inside)' => [
        'widgetSettings' => self::createWidgetSettings('de', [
          'symbol' => '€',
          'symbol_place' => 's',
          'thousand_separator' => '.',
          'decimal_separator' => ',',
          'negativeBracketsTypeOnBlur' => '(,)',
        ], ['symbolPlacementInBrackets' => 'inside']),
        'value' => -9876.54,
        'langcode' => 'de',
        'expectedOutput' => '(9.876,54 €)',
      ],
      'German Negative with Brackets (outside)' => [
        'widgetSettings' => self::createWidgetSettings('de', [
          'symbol' => '€',
          'symbol_place' => 's',
          'thousand_separator' => '.',
          'decimal_separator' => ',',
          'negativeBracketsTypeOnBlur' => '(,)',
        ], ['symbolPlacementInBrackets' => 'outside']),
        'value' => -9876.54,
        'langcode' => 'de',
        'expectedOutput' => '(9.876,54) €',
      ],
      'French Number with Positive Sign' => [
        'widgetSettings' => self::createWidgetSettings('fr', [
          'symbol' => '€',
          'symbol_place' => 's',
          'thousand_separator' => ' ',
          'decimal_separator' => ',',
          'showPositiveSign' => TRUE,
          'positiveSignCharacter' => '+',
          'negativePositiveSignPlacement' => 'p',
        ]),
        'value' => 500.00,
        'langcode' => 'fr',
        'expectedOutput' => '+500,00 €',
      ],
      'Styled Positive Number' => [
        'widgetSettings' => self::createWidgetSettings('en', [
          'symbol' => '',
          'thousand_separator' => ',',
          'decimal_separator' => '.',
        ], ['styleRules_preset' => 'positiveNegative']),
        'value' => 100,
        'langcode' => 'en',
        'expectedOutput' => '<span class="auto-numeric-positive">100.00</span>',
      ],
    ];
  }

  /**
   * Helper function to generate a valid widget settings array.
   *
   * @param string $langcode
   * The langcode to generate settings for.
   * @param array $lang_settings
   * The specific per-language settings.
   * @param array $global_settings
   * The specific global settings.
   *
   * @return array
   * A valid widget settings array.
   */
  private static function createWidgetSettings(string $langcode, array $lang_settings, array $global_settings = []): array {
    $suffixed_settings = [];
    foreach ($lang_settings as $key => $value) {
      $suffixed_settings[$key . '_' . $langcode] = $value;
    }

    return [
      'global_settings' => $global_settings,
      'language_settings' => [
        $langcode => $suffixed_settings,
      ],
    ];
  }

  /**
   * Helper function to get the default settings for a form display component.
   */
  private function getDisplayComponentDefaults(): array {
    // We must pass a valid configuration array to create an instance of the
    // widget plugin, including the field definition it applies to.
    $configuration = [
      'field_definition' => $this->fieldDefinition,
      'settings' => [],
      'third_party_settings' => [],
      // The view mode and form mode are not strictly necessary for getting
      // default settings but are good practice to include.
      'view_mode' => 'default',
      'form_mode' => 'default',
    ];

    return \Drupal::service('plugin.manager.field.widget')
      ->createInstance('formatted_number_widget', $configuration)
      ->getSettings();
  }

}