<?php

/**
* Unit tests for:
*
* Positive Numbers: US format, EU format, integers, suffix/prefix symbols. (Covered)
* Negative Numbers (Sign): Standard sign (-), alternative signs (–). (Covered)
* Negative Numbers (Brackets): Parentheses (), square brackets []. (Covered)
* Zero: (Not explicitly tested, but implied). We should add a specific case for 0.
* Formatting Edge Cases: Whitespace, no separators, symbol only. (Covered)
* Invalid/Empty Input: Empty string '', null. (Covered)
* Complex Combinations:
*  Brackets + suffix symbol + whitespace. (Covered)
*  A number that contains characters from the symbol (e.g., if the symbol was "1", testing "121" -> "2"). This is an edge case worth considering.
*  A value with only a positive or negative sign.
* # Run all tests in the 'formatted_number_input' group
* ./vendor/bin/phpunit -c web/core/phpunit.xml.dist --group formatted_number_input
* drush test:run --group formatted_number_input
*
* # Or, run the specific test file
* ./vendor/bin/phpunit -c web/core/phpunit.xml.dist web/modules/custom/formatted_number_input/tests/src/Unit/UnformatValueTest.php
* To run a specific test file by its path
* drush test:run web/modules/custom/formatted_number_input/tests/src/Unit/UnformatValueTest.php
* To run a specific test by its full class name
* drush test:run "Drupal\Tests\formatted_number_input\Unit\UnformatValueTest"
*/

/**
 * @file
 * Contains \Drupal\Tests\formatted_number_input\Unit\UnformatValueTest.
 */

namespace Drupal\Tests\formatted_number_input\Unit;

use Drupal\formatted_number_input\Plugin\Field\FieldWidget\FormattedNumberWidget;
use Drupal\Tests\UnitTestCase;

/**
 * Tests the un-formatting logic of the FormattedNumberWidget.
 *
 * @coversDefaultClass \Drupal\formatted_number_input\Plugin\Field\FieldWidget\FormattedNumberWidget
 * @group formatted_number_input
 */
class UnformatValueTest extends UnitTestCase {

  /**
   * @covers ::unformatValue
   * @dataProvider providerUnformatValue
   */
  public function testUnformatValue(?string $expected, ?string $input, array $settings, string $message): void {
    $this->assertSame($expected, FormattedNumberWidget::unformatValue($input, $settings), $message);
  }

  /**
   * Data provider for testUnformatValue().
   *
   * Each settings array now includes all keys the unformatValue function might
   * use, accurately simulating the data passed from the widget validator.
   * The test cases are systematically grouped by category.
   */
  public function providerUnformatValue(): array {
    $base_settings = [
      'language_symbol' => '',
      'thousand_separator' => '',
      'decimal_separator' => '.',
      'negativeBracketsTypeOnBlur' => 'none',
      'positiveSignCharacter' => '+',
      'negativeSignCharacter' => '-',
    ];

    $us_settings = array_merge($base_settings, [
      'language_symbol' => '$',
      'thousand_separator' => ',',
      'decimal_separator' => '.',
    ]);

    $de_settings = array_merge($base_settings, [
      'language_symbol' => '€',
      'thousand_separator' => '.',
      'decimal_separator' => ',',
    ]);

    $positive_numbers = [
      'Positive: US Currency' => [
        '1250000.75',
        '$1,250,000.75',
        $us_settings,
        'Positive US currency should be unformatted correctly.',
      ],
      'Positive: European Currency Suffix' => [
        '1250000.75',
        '1.250.000,75 €',
        $de_settings,
        'Positive European currency with suffix symbol should be unformatted correctly.',
      ],
      'Positive: Integer value' => [
        '5000',
        '5,000',
        $us_settings,
        'Positive integer with thousand separator should be unformatted correctly.',
      ],
      'Positive: Number with positive sign' => [
        '1234.56',
        '+1,234.56',
        $us_settings,
        'Positive number with explicit plus sign should be unformatted correctly.',
      ],
    ];

    $negative_numbers_sign = [
      'Negative Sign: European Currency' => [
        '-1234.56',
        '-1.234,56 €',
        $de_settings,
        'Negative European currency with sign should be unformatted correctly.',
      ],
      'Negative Sign: Alternative Negative Sign (en-dash)' => [
        '-1234.56',
        '–1,234.56',
        array_merge($us_settings, ['negativeSignCharacter' => '–']),
        'Negative number with alternative sign (en-dash) should be unformatted correctly.',
      ],
    ];

    $negative_numbers_brackets = [
      'Negative Brackets: Parentheses with prefix symbol' => [
        '-1234.56',
        '($1,234.56)',
        array_merge($us_settings, ['negativeBracketsTypeOnBlur' => '(,)']),
        'Negative number in parentheses with prefix symbol should be unformatted correctly.',
      ],
      'Negative Brackets: Square brackets with suffix symbol' => [
        '-1234.56',
        '[1.234,56 €]',
        array_merge($de_settings, ['negativeBracketsTypeOnBlur' => '[,]']),
        'Negative number in square brackets with suffix symbol should be unformatted correctly.',
      ],
      'Negative Brackets: Curly braces' => [
        '-1234.56',
        '{$1,234.56}',
        array_merge($us_settings, ['negativeBracketsTypeOnBlur' => '{,}']),
        'Negative number in curly braces should be unformatted correctly.',
      ],
    ];

    $edge_cases = [
      'Edge Case: No separators' => [
        '12345',
        '12345',
        $us_settings,
        'Number with no separators should pass through unchanged.',
      ],
      'Edge Case: Extra whitespace' => [
        '1234.56',
        ' $1,234.56 ',
        $us_settings,
        'Whitespace surrounding the formatted number should be trimmed.',
      ],
      'Edge Case: Zero' => [
        '0.00',
        '$0.00',
        $us_settings,
        'Zero value should be unformatted correctly.',
      ],
      'Edge Case: Value is only a symbol' => [
        '',
        '$',
        $us_settings,
        'Value containing only a symbol should result in an empty string.',
      ],
      'Edge Case: Value is only a sign' => [
        '',
        '+',
        $us_settings,
        'Value containing only a positive sign should result in an empty string.',
      ],
      'Edge Case: Value is only a negative sign' => [
        '-',
        '-',
        $us_settings,
        'Value containing only a negative sign should be preserved.',
      ],
    ];

    $empty_and_null = [
      'Invalid Input: Empty string' => [
        NULL,
        '',
        $base_settings,
        'Empty string input should result in NULL output.',
      ],
      'Invalid Input: Null value' => [
        NULL,
        NULL,
        $base_settings,
        'NULL input should result in NULL output.',
      ],
    ];

    return array_merge(
      $positive_numbers,
      $negative_numbers_sign,
      $negative_numbers_brackets,
      $edge_cases,
      $empty_and_null
    );
  }
}